"""Integration tests for the CpioCommand."""

from __future__ import annotations

import tempfile
from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a CpioCommand instance."""
    yield pebble_shell.commands.CpioCommand(shell=shell)


def test_name(command: pebble_shell.commands.CpioCommand):
    assert command.name == "cpio"


def test_category(command: pebble_shell.commands.CpioCommand):
    assert command.category == "Archive Utilities"


def test_help(command: pebble_shell.commands.CpioCommand):
    with command.shell.console.capture() as capture:
        command.show_help()
    output = capture.get()
    assert "cpio" in output
    assert "Archive creation and extraction" in output
    assert "archive" in output


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    output = capture.get()
    assert "cpio" in output
    assert "Archive creation and extraction" in output


def test_execute_no_args(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # cpio with no args should fail and show usage
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # Should fail with usage message
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["usage", "Usage", "cpio", "mode", "operation"])


def test_execute_list_archive_contents(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test listing archive contents
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        nonexistent_cpio = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", nonexistent_cpio])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(
        msg in output for msg in ["No such file", "cannot", "error", "not found"]
    )


def test_execute_extract_archive(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test extracting from archive
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        nonexistent_cpio = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-i", "-I", nonexistent_cpio])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(
        msg in output for msg in ["No such file", "cannot", "error", "not found"]
    )


def test_execute_create_archive_mode(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test create archive mode (dangerous, so use safe approach)
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-o", "-O", output_path])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should create archive from stdin
        assert len(output.strip()) >= 0
    else:
        # Should fail if cannot create archive
        assert result == 1


def test_execute_pass_through_mode(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test pass-through mode
    destination_dir = tempfile.mkdtemp()
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-p", destination_dir])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should copy files from stdin
        assert len(output.strip()) >= 0
    else:
        # Should fail if destination doesn't exist
        assert result == 1


def test_execute_verbose_mode(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test verbose mode
    with command.shell.console.capture() as capture:
        with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
            archive_path = archive_file.name
        result = command.execute(client=client, args=["-tv", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_create_directories(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test create directories option
    with command.shell.console.capture() as capture:
        with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
            archive_path = archive_file.name
        result = command.execute(client=client, args=["-id", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_preserve_modification_time(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test preserve modification time
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-im", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_unconditional_overwrite(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test unconditional overwrite
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-iu", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_reset_access_times(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test reset access times
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-oa", "-O", output_path])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should reset access times
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_append_mode(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test append mode
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-oA", "-O", output_path])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should append to archive
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_block_size_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test block size option
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-B", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_format_specification(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test format specification
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client, args=["-o", "-H", "newc", "-O", output_path]
        )

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should use specified format
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_pattern_matching(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test pattern matching
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client, args=["-i", "*.txt", "-I", archive_path]
        )

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_rename_files(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test rename files option
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-ir", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_swap_bytes(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test swap bytes option
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-is", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_swap_halfwords(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test swap halfwords option
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-iS", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_only_newer_files(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test extract only newer files
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-iu", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_link_files(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test link files option
    destination_dir = tempfile.mkdtemp()
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-pl", destination_dir])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should link files instead of copying
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_dereference_symlinks(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test dereference symbolic links
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-oL", "-O", output_path])

    # Should either wait for input or fail appropriately
    if result == 0:
        output = capture.get()
        # Should dereference symlinks
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_quiet_mode(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test quiet mode
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as archive_file:
        archive_path = archive_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-tq", "-I", archive_path])

    # Should fail gracefully for nonexistent archive
    assert result == 1
    # Should produce minimal output in quiet mode
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_file_list_from_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test reading file list from file
    with tempfile.NamedTemporaryFile(suffix=".list", delete=False) as list_file:
        list_path = list_file.name
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client,
            args=["-o", "-E", list_path, "-O", output_path],
        )

    # Should fail gracefully for nonexistent file list
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "cannot", "error"])


def test_execute_invalid_option_combination(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test invalid option combination
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-io"])

    # Should fail with invalid combination error
    assert result == 1
    output = capture.get()
    assert any(
        msg in output
        for msg in ["invalid", "conflicting", "error", "usage", "incompatible"]
    )


def test_execute_missing_required_argument(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test missing required argument
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-I"])

    # Should fail with missing argument error
    assert result == 1
    output = capture.get()
    assert any(
        msg in output for msg in ["argument", "required", "missing", "usage", "error"]
    )


def test_execute_invalid_format(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test invalid format specification
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
        output_path = output_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client, args=["-o", "-H", "invalid", "-O", output_path]
        )

    # Should fail with invalid format error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "format", "unknown", "error"])


def test_execute_permission_denied_archive(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test permission denied on archive
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", "/root/.ssh/id_rsa"])

    # Should either succeed or fail with permission error
    if result == 1:
        output = capture.get()
        assert any(msg in output for msg in ["permission", "denied", "cannot", "error"])
    else:
        # May succeed if file is readable
        assert result == 0


def test_execute_directory_as_archive(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test directory as archive
    with command.shell.console.capture() as capture:
        test_dir = tempfile.mkdtemp()
        result = command.execute(client=client, args=["-t", "-I", test_dir])

    # Should fail with directory error
    assert result == 1
    output = capture.get()
    assert any(
        msg in output for msg in ["directory", "Is a directory", "error", "invalid"]
    )


def test_execute_empty_archive(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test empty archive
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", "/dev/null"])

    # Should either handle empty archive or fail appropriately
    if result == 0:
        output = capture.get()
        # Should show empty archive
        assert len(output.strip()) == 0
    else:
        # Should fail with invalid format
        assert result == 1


def test_execute_binary_file_as_archive(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test binary file as archive
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", "/bin/ls"])

    # Should fail with invalid format error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "format", "error", "not a cpio"])


def test_execute_output_format_validation(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test output format validation
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should produce properly formatted output
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    # Should have valid error format
    assert len(output.strip()) > 0


def test_execute_memory_efficiency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test memory efficiency
    with command.shell.console.capture() as capture:
        with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
            output_path = output_file.name
        result = command.execute(client=client, args=["-o", "-O", output_path])

    # Should be memory efficient
    if result == 0:
        output = capture.get()
        # Should not consume excessive memory
        assert len(output) < 10000  # Reasonable output size limit
    else:
        assert result == 1


def test_execute_concurrent_execution(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test concurrent execution safety
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should handle concurrent execution safely
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    assert len(output.strip()) > 0


def test_execute_error_recovery(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test error recovery capabilities
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-io"])

    # Should recover from errors gracefully
    assert result == 1
    output = capture.get()
    # Should provide meaningful error message
    assert any(msg in output for msg in ["invalid", "error", "usage", "conflicting"])


def test_execute_signal_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test signal handling during processing
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should handle signals appropriately
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    # Should be signal-safe
    assert len(output.strip()) > 0


def test_execute_locale_independence(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test locale independence
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should work regardless of locale settings
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    # Should be locale-independent
    assert len(output.strip()) > 0


def test_execute_cross_platform_compatibility(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test cross-platform compatibility
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should work across different platforms
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    # Should handle platform-specific archive formats
    assert len(output.strip()) > 0


def test_execute_archive_format_detection(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test archive format detection
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", "/etc/passwd"])

    # Should detect invalid archive format
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "format", "error", "not a cpio"])


def test_execute_performance_optimization(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test performance optimization
    with command.shell.console.capture() as capture:
        with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as output_file:
            output_path = output_file.name
        result = command.execute(client=client, args=["-o", "-O", output_path])

    # Should complete efficiently
    if result == 0:
        output = capture.get()
        # Should process efficiently
        assert len(output) >= 0
    else:
        assert result == 1


def test_execute_data_consistency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test data consistency
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file1:
        tmp_name1 = tmp_file1.name
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file2:
        tmp_name2 = tmp_file2.name
    with command.shell.console.capture() as _:
        result1 = command.execute(client=client, args=["-t", "-I", tmp_name1])

    with command.shell.console.capture() as _:
        result2 = command.execute(client=client, args=["-t", "-I", tmp_name2])

    # Should produce consistent results
    assert result1 == 1
    assert result2 == 1
    # Both executions should fail consistently
    assert result1 == result2


def test_execute_robust_operation(
    client: ops.pebble.Client,
    command: pebble_shell.commands.CpioCommand,
):
    # Test robust operation under stress
    with tempfile.NamedTemporaryFile(suffix=".cpio", delete=False) as tmp_file:
        tmp_name = tmp_file.name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "-I", tmp_name])

    # Should operate robustly
    assert result == 1  # Fails for nonexistent file
    output = capture.get()
    # Should handle stress conditions
    assert len(output.strip()) > 0



"""Implementation of PullCommand."""

from __future__ import annotations

import os
import subprocess

import ops
import shimmer

from ...utils.command_helpers import handle_help_flag, validate_min_args
from .._base import Command


class PullCommand(Command):
    """Pull files and directories from the remote container."""

    name = "pull"
    help = "Pull files and directories from the remote container. Usage: pebble pull <source> <dest>"
    category = "Pebble Management"

    def execute(
        self, client: ops.pebble.Client | shimmer.PebbleCliClient, args: list[str]
    ) -> int:
        """Execute the pull command."""
        if handle_help_flag(self, args):
            return 0

        if not validate_min_args(
            self.shell, args, 2, "Usage: pebble pull <source> <dest>"
        ):
            return 1

        source = args[0]
        dest = args[1]

        if isinstance(client, ops.pebble.Client):
            env = os.environ.copy()
            assert isinstance(self.shell.client, ops.pebble.Client)
            env["PEBBLE_SOCKET"] = self.shell.client.socket_path
            cmd = ["pebble", "pull", source, dest]
        elif isinstance(client, shimmer.PebbleCliClient):  # type: ignore
            cmd = [client.pebble_binary, "pull", source, dest]
        else:
            self.console.print("Error: unsupported client type")
            return 1

        result = subprocess.run(  # noqa: S603
            cmd,
            capture_output=False,
            text=True,
            env=env if isinstance(client, ops.pebble.Client) else None,
        )
        return result.returncode



"""Integration tests for the IplinkCommand."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a IplinkCommand instance."""
    yield pebble_shell.commands.IplinkCommand(shell=shell)


def test_name(command: pebble_shell.commands.IplinkCommand):
    assert command.name == "ip link" or command.name == "iplink"


def test_category(command: pebble_shell.commands.IplinkCommand):
    assert command.category == "Network"


def test_help(command: pebble_shell.commands.IplinkCommand):
    with command.shell.console.capture() as capture:
        command.show_help()
    output = capture.get()
    assert any(
        phrase in output.lower()
        for phrase in ["ip", "link", "interface", "network", "device"]
    )


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    output = capture.get()
    assert any(
        phrase in output.lower() for phrase in ["ip", "link", "interface", "usage"]
    )


def test_execute_show_all_links(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test showing all network links
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed showing links or fail gracefully
    if result == 0:
        output = capture.get()
        # Should contain interface link information
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain interface information
            lines = output.split("\n")
            assert len(lines) >= 1
            # Should contain network interface indicators
            interface_found = any(
                any(
                    indicator in line
                    for indicator in [":", "mtu", "state", "lo", "eth", "wlan"]
                )
                for line in lines
                if line.strip()
            )
            if interface_found:
                assert interface_found
    else:
        # Should fail if no permission or network unavailable
        assert result == 1


def test_execute_show_specific_interface(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test showing specific interface
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show", "lo"])

    # Should either succeed showing loopback or fail gracefully
    if result == 0:
        output = capture.get()
        # Should contain loopback interface information
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain loopback information
            assert "lo" in output.lower() or "loopback" in output.lower()
    else:
        # Should fail if interface doesn't exist
        assert result == 1


def test_execute_show_up_interfaces(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test showing only up interfaces
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show", "up"])

    # Should either succeed showing up interfaces or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show only active interfaces
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain UP state interfaces
            assert "UP" in output or "up" in output.lower()
    else:
        assert result == 1


def test_execute_list_command(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test list command (alias for show)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["list"])

    # Should either succeed listing links or fail gracefully
    if result == 0:
        output = capture.get()
        # Should list all interfaces
        assert len(output) >= 0
        if len(output) > 0:
            lines = output.split("\n")
            assert len(lines) >= 1
    else:
        assert result == 1


def test_execute_set_interface_up(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test setting interface up (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["set", "lo", "up"])

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should set interface up (if permitted)
        assert len(output) >= 0
    else:
        # Should fail if not root
        assert result == 1
        output = capture.get()
        assert any(
            msg in output
            for msg in ["permission", "denied", "root", "error", "not found"]
        )


def test_execute_set_interface_down(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test setting interface down (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["set", "lo", "down"])

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should set interface down (if permitted)
        assert len(output) >= 0
    else:
        # Should fail if not root
        assert result == 1


def test_execute_set_mtu(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test setting MTU (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["set", "lo", "mtu", "1500"])

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should set MTU (if permitted)
        assert len(output) >= 0
    else:
        # Should fail if not root
        assert result == 1


def test_execute_set_mac_address(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test setting MAC address (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client, args=["set", "lo", "address", "00:11:22:33:44:55"]
        )

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should set MAC address (if permitted)
        assert len(output) >= 0
    else:
        # Should fail if not root or invalid for loopback
        assert result == 1


def test_execute_add_virtual_interface(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test adding virtual interface (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["add", "test0", "type", "dummy"])

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should add interface (if permitted)
        assert len(output) >= 0
    else:
        # Should fail if not root
        assert result == 1
        output = capture.get()
        assert any(msg in output for msg in ["permission", "denied", "root", "error"])


def test_execute_delete_interface(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test deleting interface (should require privileges)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["delete", "test0"])

    # Should either succeed or fail with permission/not found error
    if result == 0:
        output = capture.get()
        # Should delete interface (if permitted and exists)
        assert len(output) >= 0
    else:
        # Should fail if not root or interface doesn't exist
        assert result == 1


def test_execute_invalid_interface(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test with invalid interface name
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show", "nonexistent99"])

    # Should fail with interface not found error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["not exist", "not found", "invalid", "error"])


def test_execute_invalid_mtu_value(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test with invalid MTU value
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["set", "lo", "mtu", "abc"])

    # Should fail with invalid MTU error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "mtu", "number", "error"])


def test_execute_invalid_mac_address(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test with invalid MAC address format
    with command.shell.console.capture() as capture:
        result = command.execute(
            client=client, args=["set", "lo", "address", "invalid:mac"]
        )

    # Should fail with invalid address error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "address", "mac", "error"])


def test_execute_invalid_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test invalid option
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-z", "show"])

    # Should fail with invalid option error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "unknown", "option", "usage"])


def test_execute_link_state_detection(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test link state detection
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed detecting states or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show link states
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain state information
            has_state = any(
                state in output.upper()
                for state in ["UP", "DOWN", "UNKNOWN", "LOWER_UP"]
            )
            if has_state:
                assert has_state
    else:
        assert result == 1


def test_execute_mtu_information(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test MTU information display
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show", "lo"])

    # Should either succeed showing MTU or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show MTU information
        assert len(output) >= 0
        if len(output) > 0 and "mtu" in output.lower():
            # Should contain MTU information
            assert "mtu" in output.lower()
    else:
        assert result == 1


def test_execute_mac_address_display(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test MAC address display
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed showing MAC addresses or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show MAC address information
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain MAC address patterns (for non-loopback interfaces)
            import re

            mac_pattern = r"([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}"
            if re.search(mac_pattern, output):
                assert True  # MAC addresses found
    else:
        assert result == 1


def test_execute_interface_type_detection(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test interface type detection
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed detecting types or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show interface types
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain type information
            has_type = any(
                iface_type in output.lower()
                for iface_type in ["loopback", "ethernet", "wireless", "tunnel"]
            )
            if has_type:
                assert has_type
    else:
        assert result == 1


def test_execute_permission_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test permission handling for modifications
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["set", "lo", "mtu", "1400"])

    # Should either succeed or fail with permission error
    if result == 0:
        output = capture.get()
        # Should succeed if running as root
        assert len(output) >= 0
    else:
        # Should fail if not root
        assert result == 1
        output = capture.get()
        assert any(msg in output for msg in ["permission", "denied", "root", "error"])


def test_execute_network_namespace_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test network namespace handling
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed in current namespace or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show interfaces in current namespace
        assert len(output) >= 0
    else:
        # Should fail if namespace unavailable
        assert result == 1


def test_execute_container_environment(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test container environment handling
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should either succeed in container or fail gracefully
    if result == 0:
        output = capture.get()
        # Should handle container networking
        assert len(output) >= 0
    else:
        # May fail in restricted containers
        assert result == 1


def test_execute_loopback_interface_validation(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test loopback interface validation
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show", "lo"])

    # Should either succeed showing loopback or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show loopback interface
        assert len(output) >= 0
        if len(output) > 0:
            # Should contain loopback information
            assert any(indicator in output.lower() for indicator in ["lo", "loopback"])
    else:
        assert result == 1


def test_execute_memory_efficiency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test memory efficiency
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should be memory efficient
    if result == 0:
        output = capture.get()
        # Should not consume excessive memory
        assert len(output) < 100000  # Reasonable output size limit
    else:
        assert result == 1


def test_execute_error_recovery(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test error recovery capabilities
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["invalid_command"])

    # Should recover from errors gracefully
    assert result == 1
    output = capture.get()
    # Should provide meaningful error message
    assert any(
        msg in output for msg in ["invalid", "unknown", "error", "usage", "command"]
    )


def test_execute_signal_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test signal handling during link operations
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should handle signals appropriately
    if result == 0:
        output = capture.get()
        assert len(output) >= 0
    else:
        assert result == 1


def test_execute_cross_platform_compatibility(
    client: ops.pebble.Client,
    command: pebble_shell.commands.IplinkCommand,
):
    # Test cross-platform compatibility
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["show"])

    # Should work across different platforms
    if result == 0:
        output = capture.get()
        # Should handle platform differences
        assert len(output) >= 0
    else:
        # May fail on platforms with different networking
        assert result == 1



"""Unit tests for proc_reader utility module."""

from unittest.mock import MagicMock

import ops
import pytest

from pebble_shell.utils.proc_reader import (
    ProcReadError,
    get_boot_time_from_stat,
    get_group_name_for_gid,
    get_hostname_from_proc_sys,
    get_process_tty,
    get_user_name_for_uid,
    parse_network_address,
    parse_proc_arp,
    parse_proc_cpuinfo,
    parse_proc_diskstats,
    parse_proc_limits_file,
    parse_proc_loadavg,
    parse_proc_meminfo,
    parse_proc_mounts_file,
    parse_proc_net_connections,
    parse_proc_net_dev,
    parse_proc_route,
    parse_proc_stat,
    parse_proc_table,
    parse_proc_uptime,
    parse_proc_vmstat,
    parse_tcp_state,
    read_proc_environ,
    read_proc_file,
    read_proc_status_field,
    read_proc_status_fields,
)


class TestReadProcFile:
    """Tests for read_proc_file function."""

    def test_read_proc_file_success(self):
        """Test successful reading of a /proc file."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = "test content"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_file(mock_client, "/proc/version")

        assert result == "test content"
        mock_client.pull.assert_called_once_with("/proc/version")

    def test_read_proc_file_bytes_content(self):
        """Test reading file that returns bytes."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = b"test content"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_file(mock_client, "/proc/version")

        assert result == "test content"

    def test_read_proc_file_with_unicode_errors(self):
        """Test reading file with invalid UTF-8 bytes."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = b"\xff\xfe invalid utf8"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_file(mock_client, "/proc/version")

        # Should handle unicode errors gracefully
        assert "invalid utf8" in result

    def test_read_proc_file_path_error(self):
        """Test handling of PathError."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        with pytest.raises(ProcReadError) as exc_info:
            read_proc_file(mock_client, "/proc/nonexistent")

        error: ProcReadError = exc_info.value  # type: ignore[assignment]
        assert error.path == "/proc/nonexistent"
        assert "message" in str(error)

    def test_read_proc_file_api_error(self):
        """Test handling of APIError."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.APIError(
            {"error": "not found"}, 404, "Not Found", "File not found"
        )

        with pytest.raises(ProcReadError) as exc_info:
            read_proc_file(mock_client, "/proc/nonexistent")

        error: ProcReadError = exc_info.value  # type: ignore[assignment]
        assert error.path == "/proc/nonexistent"


class TestParseProcTable:
    """Tests for parse_proc_table function."""

    def test_parse_proc_table_basic(self):
        """Test basic table parsing."""
        content = """Header line
col1 col2 col3
val1 val2 val3
val4 val5 val6"""

        result = parse_proc_table(content, skip_header_lines=1)

        expected = [
            ["col1", "col2", "col3"],
            ["val1", "val2", "val3"],
            ["val4", "val5", "val6"],
        ]
        assert result == expected

    def test_parse_proc_table_no_headers(self):
        """Test parsing without skipping headers."""
        content = """val1 val2 val3
val4 val5 val6"""

        result = parse_proc_table(content, skip_header_lines=0)

        expected = [["val1", "val2", "val3"], ["val4", "val5", "val6"]]
        assert result == expected

    def test_parse_proc_table_empty_lines(self):
        """Test parsing with empty lines."""
        content = """Header

val1 val2 val3

val4 val5 val6
"""

        result = parse_proc_table(content, skip_header_lines=1)

        expected = [["val1", "val2", "val3"], ["val4", "val5", "val6"]]
        assert result == expected

    def test_parse_proc_table_multiple_headers(self):
        """Test parsing with multiple header lines."""
        content = """Header 1
Header 2
val1 val2 val3
val4 val5 val6"""

        result = parse_proc_table(content, skip_header_lines=2)

        expected = [["val1", "val2", "val3"], ["val4", "val5", "val6"]]
        assert result == expected


class TestReadProcStatusField:
    """Tests for read_proc_status_field function."""

    def test_read_proc_status_field_success(self):
        """Test successful field extraction."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Uid:	1000	1000	1000	1000
Gid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_field(mock_client, "1234", "Uid")

        assert result == "1000"
        mock_client.pull.assert_called_once_with("/proc/1234/status")

    def test_read_proc_status_field_not_found(self):
        """Test field not found."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Gid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_field(mock_client, "1234", "Uid")

        assert result is None

    def test_read_proc_status_field_empty_value(self):
        """Test field with empty value."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Uid:
Gid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_field(mock_client, "1234", "Uid")

        assert result is None

    def test_read_proc_status_field_self_pid(self):
        """Test reading from self process."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Uid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_field(mock_client, "self", "Uid")

        assert result == "1000"
        mock_client.pull.assert_called_once_with("/proc/self/status")


class TestReadProcStatusFields:
    """Tests for read_proc_status_fields function."""

    def test_read_proc_status_fields_multiple(self):
        """Test reading multiple fields."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Uid:	1000	1000	1000	1000
Gid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_fields(mock_client, "1234", ["Uid", "Gid", "Name"])

        expected = {"Uid": "1000", "Gid": "1000", "Name": "test"}
        assert result == expected

    def test_read_proc_status_fields_partial(self):
        """Test reading fields where some are missing."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Name:	test
Uid:	1000	1000	1000	1000"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_status_fields(mock_client, "1234", ["Uid", "Gid", "Name"])

        expected = {"Uid": "1000", "Name": "test"}
        assert result == expected


class TestReadProcEnviron:
    """Tests for read_proc_environ function."""

    def test_read_proc_environ_success(self):
        """Test successful environment variable reading."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = (
            "PATH=/bin:/usr/bin\x00HOME=/home/user\x00USER=testuser\x00"
        )
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_environ(mock_client, "1234")

        expected = {"PATH": "/bin:/usr/bin", "HOME": "/home/user", "USER": "testuser"}
        assert result == expected

    def test_read_proc_environ_empty_values(self):
        """Test environment variables with empty values."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = "PATH=/bin\x00EMPTY=\x00NOVALUE\x00"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_environ(mock_client, "1234")

        expected = {"PATH": "/bin", "EMPTY": "", "NOVALUE": ""}
        assert result == expected

    def test_read_proc_environ_self(self):
        """Test reading environment for self process."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = "USER=testuser\x00"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = read_proc_environ(mock_client, "self")

        assert result == {"USER": "testuser"}
        mock_client.pull.assert_called_once_with("/proc/self/environ")


class TestParseNetworkAddress:
    """Tests for parse_network_address function."""

    def test_parse_network_address_localhost(self):
        """Test parsing localhost address."""
        # 127.0.0.1:80 in little-endian hex
        hex_addr = "0100007F:0050"

        result = parse_network_address(hex_addr)

        assert result == "127.0.0.1:80"

    def test_parse_network_address_any(self):
        """Test parsing any address (0.0.0.0)."""
        hex_addr = "00000000:1F90"  # 0.0.0.0:8080

        result = parse_network_address(hex_addr)

        assert result == "0.0.0.0:8080"

    def test_parse_network_address_no_colon(self):
        """Test address without colon."""
        result = parse_network_address("invalid")

        assert result == "invalid"

    def test_parse_network_address_invalid_hex(self):
        """Test invalid hex address."""
        result = parse_network_address("ZZZZZZZZ:XXXX")

        assert result == "ZZZZZZZZ:XXXX"


class TestParseTcpState:
    """Tests for parse_tcp_state function."""

    def test_parse_tcp_state_established(self):
        """Test parsing ESTABLISHED state."""
        result = parse_tcp_state("01")
        assert result == "ESTABLISHED"

    def test_parse_tcp_state_listen(self):
        """Test parsing LISTEN state."""
        result = parse_tcp_state("0A")
        assert result == "LISTEN"

    def test_parse_tcp_state_lowercase(self):
        """Test parsing lowercase hex."""
        result = parse_tcp_state("0a")
        assert result == "LISTEN"

    def test_parse_tcp_state_unknown(self):
        """Test parsing unknown state."""
        result = parse_tcp_state("FF")
        assert result == "UNKNOWN(FF)"


class TestParseProcNetDev:
    """Tests for parse_proc_net_dev function."""

    def test_parse_proc_net_dev_success(self):
        """Test successful parsing of /proc/net/dev."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Inter-|   Receive                                                |  Transmit
 face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed
    lo:    1234       10    0    0    0     0          0         0     1234       10    0    0    0     0       0          0
  eth0:  567890     1000    1    2    0     0          0         0   789012     2000    0    1    0     0       0          0"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_net_dev(mock_client)

        expected = [
            {
                "interface": "lo",
                "rx_bytes": 1234,
                "rx_packets": 10,
                "rx_errors": 0,
                "rx_dropped": 0,
                "tx_bytes": 1234,
                "tx_packets": 10,
                "tx_errors": 0,
                "tx_dropped": 0,
            },
            {
                "interface": "eth0",
                "rx_bytes": 567890,
                "rx_packets": 1000,
                "rx_errors": 1,
                "rx_dropped": 2,
                "tx_bytes": 789012,
                "tx_packets": 2000,
                "tx_errors": 0,
                "tx_dropped": 1,
            },
        ]
        assert result == expected

    def test_parse_proc_net_dev_insufficient_stats(self):
        """Test parsing with insufficient stats columns."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Header1
Header2
    lo:    1234       10"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_net_dev(mock_client)

        assert result == []  # Should skip interfaces with insufficient data


class TestParseProcNetConnections:
    """Tests for parse_proc_net_connections function."""

    def test_parse_proc_net_connections_tcp(self):
        """Test parsing TCP connections."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """  sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode
   0: 0100007F:0050 00000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 12345 1 0000000000000000 100 0 0 10 0"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_net_connections(mock_client, "tcp")

        expected = [
            {
                "protocol": "TCP",
                "local_address": "127.0.0.1:80",
                "remote_address": "0.0.0.0:0",
                "state": "LISTEN",
            }
        ]
        assert result == expected

    def test_parse_proc_net_connections_udp(self):
        """Test parsing UDP connections."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """  sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode ref pointer drops
   0: 00000000:0035 00000000:0000 07 00000000:00000000 00:00000000 00000000     0        0 12345 2 0000000000000000 0"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_net_connections(mock_client, "udp")

        expected = [
            {
                "protocol": "UDP",
                "local_address": "0.0.0.0:53",
                "remote_address": "0.0.0.0:0",
            }
        ]
        assert result == expected

    def test_parse_proc_net_connections_unix(self):
        """Test parsing UNIX domain connections."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Num       RefCount Protocol Flags    Type St Inode Path
0000000000000000: 00000002 00000000 00010000 0001 01 12345 /tmp/socket
0000000000000001: 00000003 00000000 00000000 0002 01 12346"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_net_connections(mock_client, "unix")

        expected = [
            {
                "protocol": "UNIX",
                "type": "STREAM",
                "state": "CONNECTED",
                "path": "/tmp/socket",  # noqa: S108
            },
            {
                "protocol": "UNIX",
                "type": "DGRAM",
                "state": "CONNECTED",
                "path": "<unnamed>",
            },
        ]
        assert result == expected

    def test_parse_proc_net_connections_invalid_protocol(self):
        """Test parsing with invalid protocol."""
        mock_client = MagicMock()

        with pytest.raises(ValueError, match="Unsupported protocol"):
            parse_proc_net_connections(mock_client, "invalid")


class TestParseProcRoute:
    """Tests for parse_proc_route function."""

    def test_parse_proc_route_success(self):
        """Test successful parsing of routing table."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
eth0	00000000	0100007F	0003	0	0	100	00000000	0	0	0
lo	0000007F	00000000	0001	0	0	0	000000FF	0	0	0"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_route(mock_client)

        expected = [
            {
                "interface": "eth0",
                "destination": "0.0.0.0",  # noqa: S104 - test data for default route
                "gateway": "127.0.0.1",
                "flags": "UG",
                "metric": "100",
                "mask": "0.0.0.0",  # noqa: S104 - test data for netmask
            },
            {
                "interface": "lo",
                "destination": "127.0.0.0",
                "gateway": "0.0.0.0",  # noqa: S104 - test data for loopback route
                "flags": "U",
                "metric": "0",
                "mask": "255.0.0.0",
            },
        ]
        assert result == expected


class TestParseProcMeminfo:
    """Tests for parse_proc_meminfo function."""

    def test_parse_proc_meminfo_success(self):
        """Test successful parsing of /proc/meminfo."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        meminfo_content = """MemTotal:       16384000 kB
MemFree:         8192000 kB
MemAvailable:   12288000 kB
Buffers:          512000 kB
Cached:          2048000 kB
SwapCached:           0 kB
Active:          4096000 kB
Inactive:        1024000 kB
SwapTotal:       2048000 kB
SwapFree:        2048000 kB
Dirty:                64 kB
Writeback:             0 kB
"""
        mock_file.read.return_value = meminfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_meminfo(mock_client)

        assert result["MemTotal"] == 16384000
        assert result["MemFree"] == 8192000
        assert result["MemAvailable"] == 12288000
        assert result["Buffers"] == 512000
        assert result["Cached"] == 2048000
        assert result["SwapTotal"] == 2048000
        assert result["SwapFree"] == 2048000
        assert result["Dirty"] == 64
        assert result["Writeback"] == 0

    def test_parse_proc_meminfo_missing_kb_suffix(self):
        """Test parsing /proc/meminfo without kB suffix."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        meminfo_content = """MemTotal:       16384000
MemFree:         8192000 kB
VmallocTotal:   34359738367 kB
VmallocUsed:       0
"""
        mock_file.read.return_value = meminfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_meminfo(mock_client)

        assert result["MemTotal"] == 16384000
        assert result["MemFree"] == 8192000
        assert result["VmallocTotal"] == 34359738367
        assert result["VmallocUsed"] == 0

    def test_parse_proc_meminfo_invalid_values(self):
        """Test parsing /proc/meminfo with some invalid values."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        meminfo_content = """MemTotal:       16384000 kB
MemFree:         invalid_value kB
MemAvailable:   12288000 kB
BadLine
NoColon 123456
EmptyValue:
"""
        mock_file.read.return_value = meminfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_meminfo(mock_client)

        assert result["MemTotal"] == 16384000
        assert result["MemAvailable"] == 12288000
        assert "MemFree" not in result  # Should be skipped due to invalid value
        assert len(result) == 2  # Only valid entries

    def test_parse_proc_meminfo_error(self):
        """Test error handling when reading /proc/meminfo fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_meminfo(mock_client)

        assert "/proc/meminfo" in str(exc_info.value)


class TestParseProcUptime:
    """Tests for parse_proc_uptime function."""

    def test_parse_proc_uptime_success(self):
        """Test successful parsing of /proc/uptime."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        uptime_content = "12345.67 98765.43\n"
        mock_file.read.return_value = uptime_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_uptime(mock_client)

        assert result["uptime_seconds"] == 12345.67
        assert result["idle_seconds"] == 98765.43

    def test_parse_proc_uptime_invalid_format(self):
        """Test parsing /proc/uptime with invalid format."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        uptime_content = "invalid\n"
        mock_file.read.return_value = uptime_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_uptime(mock_client)

        assert "/proc/uptime" in str(exc_info.value)

    def test_parse_proc_uptime_error(self):
        """Test error handling when reading /proc/uptime fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_uptime(mock_client)

        assert "/proc/uptime" in str(exc_info.value)


class TestParseProcLoadavg:
    """Tests for parse_proc_loadavg function."""

    def test_parse_proc_loadavg_success(self):
        """Test successful parsing of /proc/loadavg."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        loadavg_content = "0.25 0.50 0.75 2/305 12345\n"
        mock_file.read.return_value = loadavg_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_loadavg(mock_client)

        assert result["load_1min"] == "0.25"
        assert result["load_5min"] == "0.50"
        assert result["load_15min"] == "0.75"
        assert result["running_processes"] == 2
        assert result["total_processes"] == 305
        assert result["last_pid"] == 12345

    def test_parse_proc_loadavg_invalid_processes(self):
        """Test parsing /proc/loadavg with invalid process format."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        loadavg_content = "0.25 0.50 0.75 invalid 12345\n"
        mock_file.read.return_value = loadavg_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_loadavg(mock_client)

        assert result["load_1min"] == "0.25"
        assert result["load_5min"] == "0.50"
        assert result["load_15min"] == "0.75"
        assert result["running_processes"] == 0
        assert result["total_processes"] == 0
        assert result["last_pid"] == 12345

    def test_parse_proc_loadavg_insufficient_fields(self):
        """Test parsing /proc/loadavg with insufficient fields."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        loadavg_content = "0.25 0.50\n"
        mock_file.read.return_value = loadavg_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_loadavg(mock_client)

        assert "/proc/loadavg" in str(exc_info.value)

    def test_parse_proc_loadavg_error(self):
        """Test error handling when reading /proc/loadavg fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_loadavg(mock_client)

        assert "/proc/loadavg" in str(exc_info.value)


class TestParseProcArp:
    """Tests for parse_proc_arp function."""

    def test_parse_proc_arp_success(self):
        """Test successful parsing of ARP table."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """IP address       HW type     Flags       HW address            Mask     Device
192.168.1.1      0x1         0x2         aa:bb:cc:dd:ee:ff     *        eth0
192.168.1.100    0x1         0x2         11:22:33:44:55:66     *        eth0"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_arp(mock_client)

        expected = [
            {
                "ip_address": "192.168.1.1",
                "hw_type": "0x1",
                "flags": "0x2",
                "hw_address": "aa:bb:cc:dd:ee:ff",
                "mask": "*",
                "device": "eth0",
            },
            {
                "ip_address": "192.168.1.100",
                "hw_type": "0x1",
                "flags": "0x2",
                "hw_address": "11:22:33:44:55:66",
                "mask": "*",
                "device": "eth0",
            },
        ]
        assert result == expected


class TestUserGroupLookup:
    """Tests for user and group name lookup functions."""

    def test_get_user_name_for_uid_success(self):
        """Test successful user lookup."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """root:x:0:0:root:/root:/bin/bash
daemon:x:1:1:daemon:/usr/sbin:/usr/sbin/nologin
testuser:x:1000:1000:Test User:/home/testuser:/bin/bash"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_user_name_for_uid(mock_client, "1000")

        assert result == "testuser"

    def test_get_user_name_for_uid_not_found(self):
        """Test user lookup when UID not found."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """root:x:0:0:root:/root:/bin/bash"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_user_name_for_uid(mock_client, "1000")

        assert result is None

    def test_get_user_name_for_uid_file_error(self):
        """Test user lookup when file cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        result = get_user_name_for_uid(mock_client, "1000")

        assert result is None

    def test_get_group_name_for_gid_success(self):
        """Test successful group lookup."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """root:x:0:
daemon:x:1:
testgroup:x:1000:testuser"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_group_name_for_gid(mock_client, "1000")

        assert result == "testgroup"

    def test_get_group_name_for_gid_not_found(self):
        """Test group lookup when GID not found."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = """root:x:0:"""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_group_name_for_gid(mock_client, "1000")

        assert result is None


class TestParseProcStat:
    """Tests for parse_proc_stat function."""

    def test_parse_proc_stat_success(self):
        """Test successful parsing of /proc/stat."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        stat_content = """cpu  123456 7890 98765 4321098 5432 0 1234 0 0 0
cpu0 61728 3945 49382 2160549 2716 0 617 0 0 0
cpu1 61728 3945 49383 2160549 2716 0 617 0 0 0
intr 12345678
ctxt 87654321
btime 1609459200
processes 123456
procs_running 2
procs_blocked 1
softirq 456789 123 456 789 0 321 654 987 0 147 258
"""
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_stat(mock_client)

        assert result["cpu"]["user"] == 123456
        assert result["cpu"]["nice"] == 7890
        assert result["cpu"]["system"] == 98765
        assert result["cpu"]["idle"] == 4321098
        assert result["cpu"]["iowait"] == 5432
        assert result["cpu"]["irq"] == 0
        assert result["cpu"]["softirq"] == 1234

        cpus_data = result["cpus"]
        assert isinstance(cpus_data, dict), (
            f"cpus should be dict, got {type(cpus_data)}"
        )
        assert "0" in cpus_data
        assert "1" in cpus_data
        assert cpus_data["0"]["user"] == 61728
        assert cpus_data["1"]["user"] == 61728

        assert result["system"]["intr"] == 12345678
        assert result["system"]["ctxt"] == 87654321
        assert result["system"]["btime"] == 1609459200
        assert result["system"]["processes"] == 123456
        assert result["system"]["procs_running"] == 2
        assert result["system"]["procs_blocked"] == 1

    def test_parse_proc_stat_minimal_format(self):
        """Test parsing /proc/stat with minimal fields."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        stat_content = """cpu  100 200 300 400
intr 12345
"""
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_stat(mock_client)

        assert result["cpu"]["user"] == 100
        assert result["cpu"]["nice"] == 200
        assert result["cpu"]["system"] == 300
        assert result["cpu"]["idle"] == 400
        assert result["cpu"]["iowait"] == 0  # Default value
        assert result["system"]["intr"] == 12345

    def test_parse_proc_stat_error(self):
        """Test error handling when reading /proc/stat fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_stat(mock_client)

        assert "/proc/stat" in str(exc_info.value)


class TestParseProcVmstat:
    """Tests for parse_proc_vmstat function."""

    def test_parse_proc_vmstat_success(self):
        """Test successful parsing of /proc/vmstat."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        vmstat_content = """nr_free_pages 123456
nr_zone_inactive_anon 78901
nr_zone_active_anon 234567
nr_zone_inactive_file 345678
nr_zone_active_file 456789
nr_zone_unevictable 0
nr_zone_write_pending 12
nr_mlock 0
nr_page_table_pages 1234
nr_kernel_stack 5678
pgpgin 123456789
pgpgout 987654321
pswpin 12345
pswpout 54321
pgalloc_dma 0
pgalloc_dma32 123456
pgalloc_normal 789012
pgfree 912345
"""
        mock_file.read.return_value = vmstat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_vmstat(mock_client)

        assert result["nr_free_pages"] == 123456
        assert result["nr_zone_inactive_anon"] == 78901
        assert result["pgpgin"] == 123456789
        assert result["pgpgout"] == 987654321
        assert result["pswpin"] == 12345
        assert result["pswpout"] == 54321
        assert result["pgfree"] == 912345

    def test_parse_proc_vmstat_invalid_values(self):
        """Test parsing /proc/vmstat with some invalid values."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        vmstat_content = """nr_free_pages 123456
invalid_line
nr_zone_active_anon invalid_value
pgpgin 987654
empty_value
"""
        mock_file.read.return_value = vmstat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_vmstat(mock_client)

        assert result["nr_free_pages"] == 123456
        assert result["pgpgin"] == 987654
        assert (
            "nr_zone_active_anon" not in result
        )  # Should be skipped due to invalid value
        assert len([k for k in result if k.startswith("nr_")]) == 1

    def test_parse_proc_vmstat_error(self):
        """Test error handling when reading /proc/vmstat fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_vmstat(mock_client)

        assert "/proc/vmstat" in str(exc_info.value)


class TestParseProcDiskstats:
    """Tests for parse_proc_diskstats function."""

    def test_parse_proc_diskstats_success(self):
        """Test successful parsing of /proc/diskstats."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        diskstats_content = """   8       0 sda 123456 7890 9876543 12345 56789 1011 4567890 23456 0 78901 98765
   8       1 sda1 98765 4321 8765432 9876 43210 987 3456789 18765 0 65432 87654
   8      16 sdb 11111 2222 3333333 4444 55555 666 7777777 8888 0 9999 10101
 259       0 nvme0n1 567890 12345 11111111 67890 123456 2345 22222222 134567 0 145678 278901
"""
        mock_file.read.return_value = diskstats_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_diskstats(mock_client)

        assert len(result) == 4

        # Test first device (sda)
        sda = result[0]
        assert sda["major"] == 8
        assert sda["minor"] == 0
        assert sda["device"] == "sda"
        assert sda["reads"] == 123456
        assert sda["read_merges"] == 7890
        assert sda["read_sectors"] == 9876543
        assert sda["read_time"] == 12345
        assert sda["writes"] == 56789
        assert sda["write_merges"] == 1011
        assert sda["write_sectors"] == 4567890
        assert sda["write_time"] == 23456
        assert sda["io_in_progress"] == 0
        assert sda["io_time"] == 78901
        assert sda["io_weighted_time"] == 98765

        # Test nvme device
        nvme = result[3]
        assert nvme["device"] == "nvme0n1"
        assert nvme["major"] == 259
        assert nvme["minor"] == 0

    def test_parse_proc_diskstats_insufficient_fields(self):
        """Test parsing /proc/diskstats with insufficient fields."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        diskstats_content = """   8       0 sda 123 456 789  # Only 6 fields, should be skipped
   8       1 sda1 98765 4321 8765432 9876 43210 987 3456789 18765 0 65432 87654  # 13 fields, valid
"""
        mock_file.read.return_value = diskstats_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_diskstats(mock_client)

        # Should only have one entry (sda1) since sda has insufficient fields
        assert len(result) == 1
        assert result[0]["device"] == "sda1"

    def test_parse_proc_diskstats_invalid_values(self):
        """Test parsing /proc/diskstats with invalid numeric values."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        diskstats_content = """   8       0 sda invalid 7890 9876543 12345 56789 1011 4567890 23456 0 78901 98765
   8       1 sda1 98765 4321 8765432 9876 43210 987 3456789 18765 0 65432 87654
"""
        mock_file.read.return_value = diskstats_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_diskstats(mock_client)

        # Should only have sda1 since sda has invalid numeric value
        assert len(result) == 1
        assert result[0]["device"] == "sda1"

    def test_parse_proc_diskstats_empty_lines(self):
        """Test parsing /proc/diskstats with empty lines."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        diskstats_content = """
   8       0 sda 123456 7890 9876543 12345 56789 1011 4567890 23456 0 78901 98765

   8       1 sda1 98765 4321 8765432 9876 43210 987 3456789 18765 0 65432 87654
"""
        mock_file.read.return_value = diskstats_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_diskstats(mock_client)

        assert len(result) == 2
        assert result[0]["device"] == "sda"
        assert result[1]["device"] == "sda1"

    def test_parse_proc_diskstats_error(self):
        """Test error handling when reading /proc/diskstats fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_diskstats(mock_client)

        assert "/proc/diskstats" in str(exc_info.value)


class TestParseProcCpuinfo:
    """Tests for parse_proc_cpuinfo function."""

    def test_parse_proc_cpuinfo_success(self):
        """Test successful parsing of /proc/cpuinfo."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        cpuinfo_content = """processor	: 0
vendor_id	: GenuineIntel
cpu family	: 6
model		: 142
model name	: Intel(R) Core(TM) i7-8650U CPU @ 1.90GHz
stepping	: 10
microcode	: 0xf0
cpu MHz		: 2100.000
cache size	: 8192 KB
physical id	: 0
siblings	: 8
core id		: 0
cpu cores	: 4
apicid		: 0
initial apicid	: 0
fpu		: yes
fpu_exception	: yes
cpuid level	: 22
wp		: yes
flags		: fpu vme de pse tsc msr pae mce cx8
bugs		:
bogomips	: 4200.00
clflush size	: 64
cache_alignment	: 64
address sizes	: 39 bits physical, 48 bits virtual
power management:

processor	: 1
vendor_id	: GenuineIntel
cpu family	: 6
model		: 142
model name	: Intel(R) Core(TM) i7-8650U CPU @ 1.90GHz
stepping	: 10
microcode	: 0xf0
cpu MHz		: 2100.000
cache size	: 8192 KB
physical id	: 0
siblings	: 8
core id		: 1
cpu cores	: 4
apicid		: 2
initial apicid	: 2
fpu		: yes
fpu_exception	: yes
cpuid level	: 22
wp		: yes
flags		: fpu vme de pse tsc msr pae mce cx8
bugs		:
bogomips	: 4200.00
clflush size	: 64
cache_alignment	: 64
address sizes	: 39 bits physical, 48 bits virtual
power management:

"""
        mock_file.read.return_value = cpuinfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_cpuinfo(mock_client)

        assert result["cpu_count"] == 2
        cpus_data = result["cpus"]
        assert isinstance(cpus_data, list), (
            f"cpus should be list, got {type(cpus_data)}"
        )
        assert len(cpus_data) == 2

        # Test first CPU
        cpu0 = cpus_data[0]
        assert cpu0["processor"] == "0"
        assert cpu0["vendor_id"] == "GenuineIntel"
        assert cpu0["model name"] == "Intel(R) Core(TM) i7-8650U CPU @ 1.90GHz"
        assert cpu0["cpu cores"] == "4"
        assert cpu0["cache size"] == "8192 KB"

        # Test second CPU
        cpu1 = cpus_data[1]
        assert cpu1["processor"] == "1"
        assert cpu1["core id"] == "1"
        assert cpu1["apicid"] == "2"

    def test_parse_proc_cpuinfo_single_cpu(self):
        """Test parsing /proc/cpuinfo with single CPU."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        cpuinfo_content = """processor	: 0
vendor_id	: AuthenticAMD
cpu family	: 23
model		: 49
model name	: AMD EPYC 7302P 16-Core Processor
stepping	: 0
microcode	: 0x8301038
cpu MHz		: 3000.000
cache size	: 512 KB
physical id	: 0
siblings	: 32
core id		: 0
cpu cores	: 16
apicid		: 0
"""
        mock_file.read.return_value = cpuinfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_cpuinfo(mock_client)

        assert result["cpu_count"] == 1
        cpus_data = result["cpus"]
        assert isinstance(cpus_data, list), (
            f"cpus should be list, got {type(cpus_data)}"
        )
        assert len(cpus_data) == 1
        assert cpus_data[0]["model name"] == "AMD EPYC 7302P 16-Core Processor"
        assert cpus_data[0]["cpu cores"] == "16"

    def test_parse_proc_cpuinfo_malformed_lines(self):
        """Test parsing /proc/cpuinfo with malformed lines."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        cpuinfo_content = """processor	: 0
vendor_id	: GenuineIntel
invalid_line_without_colon
model name	: Intel CPU
: value_without_key
processor	: 1
model name	: Intel CPU 2
"""
        mock_file.read.return_value = cpuinfo_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_cpuinfo(mock_client)

        assert result["cpu_count"] == 2
        cpus_data = result["cpus"]
        assert isinstance(cpus_data, list), (
            f"cpus should be list, got {type(cpus_data)}"
        )
        assert len(cpus_data) == 2
        assert cpus_data[0]["model name"] == "Intel CPU"
        assert cpus_data[1]["model name"] == "Intel CPU 2"

    def test_parse_proc_cpuinfo_error(self):
        """Test error handling when reading /proc/cpuinfo fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            parse_proc_cpuinfo(mock_client)

        assert "/proc/cpuinfo" in str(exc_info.value)


class TestProcReadError:
    """Tests for ProcReadError exception."""

    def test_proc_read_error_attributes(self):
        """Test ProcReadError has correct attributes."""
        error = ProcReadError("/proc/test", "test message")

        assert error.path == "/proc/test"
        assert error.message == "test message"
        assert str(error) == "Error reading /proc/test: test message"


class TestGetProcessTty:
    """Tests for get_process_tty function."""

    def test_get_process_tty_with_tty(self):
        """Test getting TTY for a process with TTY."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        # TTY field (7th) is "34816" - should convert to pts/0
        stat_content = "1234 (bash) S 1233 1234 34816 34816 1234 4194304 123 456 0 0 10 20 0 0 20 0 1 0 123456 12345678 890 18446744073709551615 4194304 4456789 140736123456 0 0 0 65536 3670016 1266777851 0 0 0 17 1 0 0 0 0 0"
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_process_tty(mock_client, "1234")

        assert result == "pts/0"
        mock_client.pull.assert_called_once_with("/proc/1234/stat")

    def test_get_process_tty_no_tty(self):
        """Test getting TTY for a process without TTY."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        # TTY field (7th) is "0" - should return "?"
        stat_content = "1234 (daemon) S 1 1234 0 0 1234 4194304 123 456 0 0 10 20 0 0 20 0 1 0 123456 12345678 890 18446744073709551615 4194304 4456789 140736123456 0 0 0 0 0 0 0 0 0 17 1 0 0 0 0 0"
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_process_tty(mock_client, "1234")

        assert result == "?"

    def test_get_process_tty_insufficient_fields(self):
        """Test getting TTY with insufficient fields in stat file."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        # Only 5 fields - should return "?"
        stat_content = "1234 (test) S 1 1234"
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_process_tty(mock_client, "1234")

        assert result == "?"

    def test_get_process_tty_read_error(self):
        """Test getting TTY when stat file read fails."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "Permission denied")

        with pytest.raises(ProcReadError) as exc_info:
            get_process_tty(mock_client, "1234")

        assert "/proc/1234/stat" in str(exc_info.value)


class TestGetBootTimeFromStat:
    """Tests for get_boot_time_from_stat function."""

    def test_get_boot_time_success(self):
        """Test successful boot time extraction."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        stat_content = """cpu  123 456 789 0 0 0 0 0 0 0
btime 1609459200
ctxt 123456
"""
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_boot_time_from_stat(mock_client)
        assert result == 1609459200

    def test_get_boot_time_no_btime(self):
        """Test when btime line is missing."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        stat_content = """cpu  123 456 789 0 0 0 0 0 0 0
ctxt 123456
"""
        mock_file.read.return_value = stat_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        with pytest.raises(ProcReadError):
            get_boot_time_from_stat(mock_client)

    def test_get_boot_time_read_error(self):
        """Test when /proc/stat cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        with pytest.raises(ProcReadError):
            get_boot_time_from_stat(mock_client)


class TestParseProcLimitsFile:
    """Tests for parse_proc_limits_file function."""

    def test_parse_limits_success(self):
        """Test successful limits parsing."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        limits_content = """Limit                     Soft Limit           Hard Limit           Units
Max cpu time              unlimited            unlimited            seconds
Max file size             unlimited            unlimited            bytes
Max data size             unlimited            unlimited            bytes
"""
        mock_file.read.return_value = limits_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_limits_file(mock_client, "self")

        assert "Max cpu time" in result
        assert result["Max cpu time"]["soft"] == "unlimited"
        assert result["Max cpu time"]["hard"] == "unlimited"
        assert result["Max cpu time"]["units"] == "seconds"

    def test_parse_limits_empty_file(self):
        """Test parsing empty limits file."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = ""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        with pytest.raises(ProcReadError):
            parse_proc_limits_file(mock_client, "self")

    def test_parse_limits_read_error(self):
        """Test when limits file cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        with pytest.raises(ProcReadError):
            parse_proc_limits_file(mock_client, "self")


class TestParseProcMountsFile:
    """Tests for parse_proc_mounts_file function."""

    def test_parse_mounts_success(self):
        """Test successful mounts parsing."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mounts_content = """sysfs /sys sysfs rw,nosuid,nodev,noexec,relatime 0 0
proc /proc proc rw,nosuid,nodev,noexec,relatime 0 0
/dev/sda1 / ext4 rw,relatime,errors=remount-ro 0 0
"""
        mock_file.read.return_value = mounts_content
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_mounts_file(mock_client)

        assert len(result) == 3
        assert result[0]["device"] == "sysfs"
        assert result[0]["mountpoint"] == "/sys"
        assert result[0]["fstype"] == "sysfs"
        assert result[0]["options"] == "rw,nosuid,nodev,noexec,relatime"

    def test_parse_mounts_empty_file(self):
        """Test parsing empty mounts file."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = ""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = parse_proc_mounts_file(mock_client)
        assert result == []

    def test_parse_mounts_read_error(self):
        """Test when mounts file cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        with pytest.raises(ProcReadError):
            parse_proc_mounts_file(mock_client)


class TestGetHostnameFromProcSys:
    """Tests for get_hostname_from_proc_sys function."""

    def test_get_hostname_success(self):
        """Test successful hostname reading."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = "test-host\n"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        result = get_hostname_from_proc_sys(mock_client)
        assert result == "test-host"

    def test_get_hostname_read_error(self):
        """Test when hostname file cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        with pytest.raises(ProcReadError):
            get_hostname_from_proc_sys(mock_client)


class TestReadProcCmdline:
    """Tests for read_proc_cmdline function."""

    def test_read_cmdline_success(self):
        """Test successful cmdline reading with null bytes."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = (
            "python3\x00/usr/bin/script.py\x00--arg1\x00--arg2\x00"
        )
        mock_client.pull.return_value.__enter__.return_value = mock_file

        from src.pebble_shell.utils.proc_reader import read_proc_cmdline

        result = read_proc_cmdline(mock_client, "1234")
        assert result == "python3 /usr/bin/script.py --arg1 --arg2"

    def test_read_cmdline_empty(self):
        """Test reading empty cmdline."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = ""
        mock_client.pull.return_value.__enter__.return_value = mock_file

        from src.pebble_shell.utils.proc_reader import read_proc_cmdline

        result = read_proc_cmdline(mock_client, "1234")
        assert result == "unknown"

    def test_read_cmdline_read_error(self):
        """Test when cmdline file cannot be read."""
        mock_client = MagicMock()
        mock_client.pull.side_effect = ops.pebble.PathError("kind", "message")

        from src.pebble_shell.utils.proc_reader import read_proc_cmdline

        result = read_proc_cmdline(mock_client, "1234")
        assert result == "unknown"

    def test_read_cmdline_self_process(self):
        """Test reading cmdline for self process."""
        mock_client = MagicMock()
        mock_file = MagicMock()
        mock_file.read.return_value = "python\x00-m\x00pytest\x00"
        mock_client.pull.return_value.__enter__.return_value = mock_file

        from src.pebble_shell.utils.proc_reader import read_proc_cmdline

        result = read_proc_cmdline(mock_client, "self")
        assert result == "python -m pytest"



"""File operation utilities for safe file handling with Pebble clients."""

from __future__ import annotations

import os
import pathlib
from typing import TYPE_CHECKING

import ops

if TYPE_CHECKING:
    import shimmer
    from rich.console import Console
    from rich.progress import Progress

    PebbleClient = ops.pebble.Client | shimmer.PebbleCliClient


def safe_pull_file(client: PebbleClient, path: str) -> str | bytes | None:
    """Safely pull a file with proper error handling.

    Args:
        client: Pebble client
        path: Path to file to pull

    Returns:
        File content as string or bytes, or None if error
    """
    try:
        with client.pull(path) as file:
            return file.read()
    except (ops.pebble.PathError, ops.pebble.APIError):
        return None


def safe_push_file(
    client: PebbleClient, path: str, content: str | bytes, make_dirs: bool = True
) -> bool:
    """Safely push a file with proper error handling.

    Args:
        client: Pebble client
        path: Destination path
        content: Content to write
        make_dirs: Whether to create parent directories

    Returns:
        True if successful, False otherwise
    """
    try:
        client.push(path, content, make_dirs=make_dirs)
        return True
    except (ops.pebble.PathError, ops.pebble.APIError):
        return False


def list_directory_safe(
    client: PebbleClient, path: str
) -> list[ops.pebble.FileInfo] | None:
    """List directory contents with error handling.

    Args:
        client: Pebble client
        path: Directory path to list

    Returns:
        List of FileInfo objects, or None if error
    """
    try:
        return client.list_files(path)
    except (ops.pebble.PathError, ops.pebble.APIError):
        return None


def file_exists(client: PebbleClient, path: str) -> bool:
    """Check if a file or directory exists.

    Args:
        client: Pebble client
        path: Path to check

    Returns:
        True if file exists, False otherwise
    """
    try:
        # Try to get the parent directory and check if file exists in it
        path_obj = pathlib.Path(path)
        parent = str(path_obj.parent) if path_obj.parent != path_obj else "."
        filename = path_obj.name

        files = client.list_files(parent)
        return any(f.name == filename for f in files)
    except (ops.pebble.PathError, ops.pebble.APIError):
        return False


def get_file_info(client: PebbleClient, path: str) -> ops.pebble.FileInfo | None:
    """Get file information for a specific path.

    Args:
        client: Pebble client
        path: Path to get info for

    Returns:
        FileInfo object if found, None otherwise
    """
    try:
        path_obj = pathlib.Path(path)
        parent = str(path_obj.parent) if path_obj.parent != path_obj else "."
        filename = path_obj.name

        files = client.list_files(parent)
        for file_info in files:
            if file_info.name == filename:
                return file_info
        return None
    except (ops.pebble.PathError, ops.pebble.APIError):
        return None


def copy_file_with_progress(
    client: PebbleClient,
    console: Console,
    source: str,
    dest: str,
    progress: Progress | None = None,
    task_id: int | None = None,
) -> bool:
    """Copy a single file with optional progress tracking.

    Args:
        client: Pebble client
        console: Console for output
        source: Source file path
        dest: Destination file path
        progress: Optional Progress instance
        task_id: Optional task ID for progress tracking

    Returns:
        True if successful, False otherwise
    """
    try:
        try:
            with client.pull(source) as file:
                content = file.read()
        except (ops.pebble.PathError, ops.pebble.APIError):
            console.print(f"cannot read source file: {source}")
            return False

        if not safe_push_file(client, dest, content, make_dirs=True):
            console.print(f"cannot write destination file: {dest}")
            return False

        console.print(f"'{source}' -> '{dest}'")

        if progress and task_id is not None:
            progress.advance(task_id)

        return True

    except Exception as e:
        console.print(f"cannot copy file: {e}")
        return False


def copy_directory_recursive(
    client: PebbleClient,
    console: Console,
    source: str,
    dest: str,
    progress: Progress | None = None,
    task_id: int | None = None,
) -> bool:
    """Copy a directory recursively with optional progress tracking.

    Args:
        client: Pebble client
        console: Console for output
        source: Source directory path
        dest: Destination directory path
        progress: Optional Progress instance
        task_id: Optional task ID for progress tracking

    Returns:
        True if successful, False otherwise
    """
    try:
        client.make_dir(dest, make_parents=True)
        console.print(f"'{source}' -> '{dest}' (directory)")

        files = list_directory_safe(client, source)
        if files is None:
            console.print(f"cannot list directory: {source}")
            return False

        success = True
        for file_info in files:
            src_path = os.path.join(source, file_info.name)
            dst_path = os.path.join(dest, file_info.name)

            if file_info.type == ops.pebble.FileType.FILE:
                if not copy_file_with_progress(
                    client, console, src_path, dst_path, progress, task_id
                ):
                    success = False
            elif (
                file_info.type == ops.pebble.FileType.DIRECTORY
                and not copy_directory_recursive(
                    client, console, src_path, dst_path, progress, task_id
                )
            ):
                success = False

        return success

    except Exception as e:
        console.print(f"cannot copy directory: {e}")
        return False


def remove_file_recursive(
    client: PebbleClient,
    console: Console,
    path: str,
    force: bool = False,
    progress: Progress | None = None,
    task_id: int | None = None,
) -> bool:
    """Remove files or directories recursively with proper error handling.

    Args:
        client: Pebble client
        console: Console for output
        path: Path to remove
        force: Whether to ignore errors
        progress: Optional Progress instance
        task_id: Optional task ID for progress tracking

    Returns:
        True if successful, False otherwise
    """
    try:
        file_info = get_file_info(client, path)
        if file_info is None:
            if not force:
                console.print(f"cannot remove '{path}': file not found")
                return False
            return True

        if file_info.type == ops.pebble.FileType.DIRECTORY:
            # Remove directory contents first.
            files = list_directory_safe(client, path)
            if files is not None:
                for sub_file in files:
                    sub_path = os.path.join(path, sub_file.name)
                    if (
                        not remove_file_recursive(
                            client, console, sub_path, force, progress, task_id
                        )
                        and not force
                    ):
                        return False

        # Remove the item itself.
        try:
            client.remove_path(path)
            console.print(f"removed '{path}'")

            if progress and task_id is not None:
                progress.advance(task_id)

            return True

        except (ops.pebble.PathError, ops.pebble.APIError) as e:
            if not force:
                console.print(f"cannot remove '{path}': {e}")
                return False
            return True

    except Exception as e:
        if not force:
            console.print(f"cannot remove '{path}': {e}")
            return False
        return True


def move_file_with_progress(
    client: PebbleClient,
    console: Console,
    source: str,
    dest: str,
    progress: Progress | None = None,
    task_id: int | None = None,
) -> bool:
    """Move a file or directory with optional progress tracking.

    This implements move as copy + delete since Pebble doesn't have native move.

    Args:
        client: Pebble client
        console: Console for output
        source: Source path
        dest: Destination path
        progress: Optional Progress instance
        task_id: Optional task ID for progress tracking

    Returns:
        True if successful, False otherwise
    """
    file_info = get_file_info(client, source)
    if file_info is None:
        console.print(f"cannot stat '{source}': file not found")
        return False

    # Copy to destination.
    if file_info.type == ops.pebble.FileType.FILE:
        if not copy_file_with_progress(
            client, console, source, dest, progress, task_id
        ):
            return False
    elif file_info.type == ops.pebble.FileType.DIRECTORY:
        if not copy_directory_recursive(
            client, console, source, dest, progress, task_id
        ):
            return False
    else:
        console.print(f"'{source}': unsupported file type")
        return False

    # Remove source.
    if not remove_file_recursive(client, console, source, force=False):
        console.print(f"moved to '{dest}' but failed to remove source '{source}'")
        return False

    return True


def ensure_parent_directory(client: PebbleClient, file_path: str) -> bool:
    """Ensure the parent directory of a file path exists.

    Args:
        client: Pebble client
        file_path: File path whose parent should exist

    Returns:
        True if parent exists or was created, False otherwise
    """
    try:
        parent = os.path.dirname(file_path)
        if parent and parent != file_path:
            client.make_dir(parent, make_parents=True)
        return True
    except (ops.pebble.PathError, ops.pebble.APIError):
        return False


def get_directory_size(client: PebbleClient, path: str) -> int:
    """Get the total size of files in a directory recursively.

    Args:
        client: Pebble client
        path: Directory path

    Returns:
        Total size in bytes, or 0 if error
    """
    total_size = 0

    try:
        files = list_directory_safe(client, path)
        if files is None:
            return 0

        for file_info in files:
            if file_info.type == ops.pebble.FileType.FILE:
                total_size += file_info.size or 0
            elif file_info.type == ops.pebble.FileType.DIRECTORY:
                sub_path = os.path.join(path, file_info.name)
                total_size += get_directory_size(client, sub_path)

    except (ops.pebble.PathError, ops.pebble.APIError):
        pass

    return total_size


def count_files_recursive(client: PebbleClient, path: str) -> int:
    """Count the total number of files in a directory recursively.

    Args:
        client: Pebble client
        path: Directory path

    Returns:
        Total file count, or 0 if error
    """
    count = 0

    try:
        files = list_directory_safe(client, path)
        if files is None:
            return 0

        for file_info in files:
            if file_info.type == ops.pebble.FileType.FILE:
                count += 1
            elif file_info.type == ops.pebble.FileType.DIRECTORY:
                sub_path = os.path.join(path, file_info.name)
                count += count_files_recursive(client, sub_path)

    except (ops.pebble.PathError, ops.pebble.APIError):
        pass

    return count



"""Integration tests for the 'pwd' command."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a PwdCommand instance."""
    yield pebble_shell.commands.PwdCommand(shell=shell)


def test_name(command: pebble_shell.commands.PwdCommand):
    assert command.name == "pwd"


def test_category(command: pebble_shell.commands.PwdCommand):
    assert command.category == "Built-in Commands"


def test_help(command: pebble_shell.commands.PwdCommand):
    command.show_help()
    with command.shell.console.capture() as capture:
        command.show_help()
    assert "Print current directory" in capture.get()


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PwdCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    assert "Print current directory" in capture.get()


@pytest.mark.parametrize("current_dir", ["/home/ubuntu", "/var/log", "/var", "/"])
def test_execute_no_args(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PwdCommand,
    current_dir: str,
):
    with command.shell.console.capture() as capture:
        command.shell.current_directory = current_dir
        result = command.execute(client=client, args=[])
    assert result == 0
    assert capture.get().strip() == current_dir


def test_execute_with_args(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PwdCommand,
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["extra_arg"])
    assert result == 1
    assert "Error: This command does not accept any arguments" in capture.get()



"""Integration tests for the OdCommand."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a OdCommand instance."""
    yield pebble_shell.commands.OdCommand(shell=shell)


def test_name(command: pebble_shell.commands.OdCommand):
    assert command.name == "od"


def test_category(command: pebble_shell.commands.OdCommand):
    assert command.category == "Data Processing"


def test_help(command: pebble_shell.commands.OdCommand):
    with command.shell.console.capture() as capture:
        command.show_help()
    output = capture.get()
    assert "od" in output
    assert "Dump files in octal format" in output
    assert "-t" in output
    assert "-x" in output


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    output = capture.get()
    assert "od" in output
    assert "Dump files in octal format" in output


def test_execute_no_args_stdin(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # od with no args should read from stdin
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # Should either succeed reading stdin or fail gracefully
    if result == 0:
        output = capture.get()
        # Should handle stdin input
        assert len(output.strip()) >= 0
    else:
        # Should fail if no stdin available
        assert result == 1


def test_execute_single_file_default_format(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test dumping single file in default octal format
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hosts"])

    # Should either succeed showing octal dump or fail if not accessible
    if result == 0:
        output = capture.get()
        # Should show octal dump with addresses and data
        assert len(output.strip()) > 0
        # Should contain octal values (digits 0-7)
        assert any(char in "01234567" for char in output)
    else:
        # Should fail if file not accessible
        assert result == 1
        output = capture.get()
        assert any(
            msg in output for msg in ["No such file", "permission denied", "error"]
        )


def test_execute_nonexistent_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with nonexistent file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/nonexistent/file.txt"])

    # Should fail with file not found error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "not found", "error"])


def test_execute_empty_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with empty file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/dev/null"])

    # Should either succeed with empty output or fail gracefully
    if result == 0:
        output = capture.get()
        # Should handle empty file (may show just final address)
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_binary_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with binary file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/bin/ls"])

    # Should succeed showing binary dump
    if result == 0:
        output = capture.get()
        # Should show binary data in octal format
        assert len(output.strip()) > 0
        # Should contain octal values
        assert any(char in "01234567" for char in output)
    else:
        assert result == 1


def test_execute_text_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with text file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/passwd"])

    # Should succeed showing text dump
    if result == 0:
        output = capture.get()
        # Should show text data in octal format
        assert len(output.strip()) > 0
        # Should contain octal values
        assert any(char in "01234567" for char in output)
    else:
        assert result == 1


def test_execute_hexadecimal_format(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -x option for hexadecimal format
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-x", "/etc/hosts"])

    # Should either succeed with hex format or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show hexadecimal dump
        assert len(output.strip()) > 0
        # Should contain hex values (0-9, a-f)
        assert any(char in "0123456789abcdef" for char in output.lower())
    else:
        assert result == 1


def test_execute_decimal_format(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -d option for decimal format
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-d", "/etc/hosts"])

    # Should either succeed with decimal format or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show decimal dump
        assert len(output.strip()) > 0
        # Should contain decimal values
        assert any(char.isdigit() for char in output)
    else:
        assert result == 1


def test_execute_character_format(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -c option for character format
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-c", "/etc/hosts"])

    # Should either succeed with character format or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show character dump with escape sequences
        assert len(output.strip()) > 0
        # Should contain character representations
        assert any(char.isprintable() for char in output)
    else:
        assert result == 1


def test_execute_type_format_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -t option with format specification
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "x1", "/etc/hosts"])

    # Should either succeed with specified format or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show dump in specified format
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_address_radix_decimal(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -A d option for decimal address radix
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-A", "d", "/etc/hosts"])

    # Should either succeed with decimal addresses or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show decimal addresses
        assert len(output.strip()) > 0
        # Should contain decimal address values
        assert any(char.isdigit() for char in output)
    else:
        assert result == 1


def test_execute_address_radix_hex(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -A x option for hexadecimal address radix
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-A", "x", "/etc/hosts"])

    # Should either succeed with hex addresses or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show hexadecimal addresses
        assert len(output.strip()) > 0
        # Should contain hex address values
        assert any(char in "0123456789abcdef" for char in output.lower())
    else:
        assert result == 1


def test_execute_address_radix_octal(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -A o option for octal address radix
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-A", "o", "/etc/hosts"])

    # Should either succeed with octal addresses or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show octal addresses
        assert len(output.strip()) > 0
        # Should contain octal address values
        assert any(char in "01234567" for char in output)
    else:
        assert result == 1


def test_execute_address_radix_none(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -A n option for no address display
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-A", "n", "/etc/hosts"])

    # Should either succeed without addresses or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show data without addresses
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_skip_bytes_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -j option to skip bytes
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-j", "10", "/etc/hosts"])

    # Should either succeed skipping bytes or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show dump starting from offset
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_read_bytes_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -N option to limit bytes read
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-N", "50", "/etc/hosts"])

    # Should either succeed reading limited bytes or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show limited amount of data
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_width_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test -w option for output width
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-w", "16", "/etc/hosts"])

    # Should either succeed with specified width or fail gracefully
    if result == 0:
        output = capture.get()
        # Should format output with specified width
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_multiple_files(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with multiple files
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hosts", "/etc/passwd"])

    # Should either succeed dumping multiple files or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show dumps from multiple files
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_large_file_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with large file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-N", "1000", "/usr/bin/bash"])

    # Should either succeed with large file or fail gracefully
    if result == 0:
        output = capture.get()
        # Should handle large file efficiently
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_small_file_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with very small file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hostname"])

    # Should either succeed with small file or fail if not found
    if result == 0:
        output = capture.get()
        # Should handle small file appropriately
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_special_files_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with special files
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-N", "100", "/proc/version"])

    # Should either succeed with special file or fail gracefully
    if result == 0:
        output = capture.get()
        # Should show special file contents in dump format
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_permission_denied_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with permission denied file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/shadow"])

    # Should fail with permission error
    if result == 1:
        output = capture.get()
        assert any(
            msg in output for msg in ["permission denied", "cannot open", "error"]
        )
    else:
        # May succeed if file is readable
        assert result == 0


def test_execute_directory_argument(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with directory argument
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc"])

    # Should either handle directory or fail appropriately
    if result == 0:
        output = capture.get()
        # Should show directory contents or error
        assert len(output.strip()) >= 0
    else:
        # Should fail if directories not supported
        assert result == 1
        output = capture.get()
        assert any(msg in output for msg in ["is a directory", "cannot open", "error"])


def test_execute_symlink_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test with symbolic link
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/bin"])  # Often a symlink

    # Should either follow symlink or handle appropriately
    if result == 0:
        output = capture.get()
        # Should follow symlink and dump target
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_address_format_consistency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test address format consistency
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hosts"])

    # Should format addresses consistently
    if result == 0:
        output = capture.get()
        # Should have consistent address formatting
        lines = output.strip().split("\n")
        if len(lines) > 1:
            # Should maintain address column alignment
            assert len(lines) > 0
    else:
        assert result == 1


def test_execute_data_format_consistency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test data format consistency
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-x", "/etc/hosts"])

    # Should format data consistently
    if result == 0:
        output = capture.get()
        # Should have consistent data formatting
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_output_alignment(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test output column alignment
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hosts"])

    # Should align output columns properly
    if result == 0:
        output = capture.get()
        # Should maintain column alignment
        lines = output.strip().split("\n")
        if len(lines) > 1:
            # Should have consistent line structure
            assert len(lines) > 0
    else:
        assert result == 1


def test_execute_byte_order_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test byte order handling
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-t", "x2", "/etc/hosts"])

    # Should handle byte order appropriately
    if result == 0:
        output = capture.get()
        # Should display multi-byte values correctly
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_unicode_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test unicode character handling
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-c", "/etc/hosts"])

    # Should handle unicode characters appropriately
    if result == 0:
        output = capture.get()
        # Should display unicode content safely
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_control_character_display(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test control character display
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-c", "/etc/hosts"])

    # Should display control characters appropriately
    if result == 0:
        output = capture.get()
        # Should show control characters with escape sequences
        assert len(output.strip()) > 0
    else:
        assert result == 1


def test_execute_null_byte_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test null byte handling
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/dev/null"])

    # Should handle null bytes appropriately
    if result == 0:
        output = capture.get()
        # Should display null bytes correctly
        assert len(output.strip()) >= 0
    else:
        assert result == 1


def test_execute_invalid_option_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test handling of invalid options
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-Z", "/etc/hosts"])

    # Should handle invalid options appropriately
    if result == 1:
        output = capture.get()
        # Should show error for invalid option
        assert any(msg in output for msg in ["invalid", "unknown", "option"])
    else:
        # May ignore unknown options
        assert result == 0


def test_execute_performance_large_dump(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test performance with large dumps
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-N", "10000", "/usr/bin/bash"])

    # Should handle large dumps efficiently
    if result == 0:
        output = capture.get()
        # Should complete in reasonable time
        assert len(output) >= 0
    else:
        assert result == 1


def test_execute_memory_efficiency(
    client: ops.pebble.Client,
    command: pebble_shell.commands.OdCommand,
):
    # Test memory efficiency
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/hosts"])

    # Should be memory efficient
    if result == 0:
        output = capture.get()
        # Should not consume excessive memory
        assert len(output) < 1000000  # Reasonable output size limit
    else:
        assert result == 1



"""Integration tests for the SplitCommand."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a SplitCommand instance."""
    yield pebble_shell.commands.SplitCommand(shell=shell)


def test_name(command: pebble_shell.commands.SplitCommand):
    assert command.name == "split"


def test_category(command: pebble_shell.commands.SplitCommand):
    assert command.category == "Data Processing"


def test_help(command: pebble_shell.commands.SplitCommand):
    with command.shell.console.capture() as capture:
        command.show_help()
    output = capture.get()
    assert "split" in output
    assert "Split a file into pieces" in output
    assert "-l NUMBER" in output
    assert "-b SIZE" in output


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    output = capture.get()
    assert "split" in output
    assert "Split a file into pieces" in output


def test_execute_no_args_default_behavior(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # split with no args should read from stdin
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=[])

    # Should either succeed (if stdin available) or fail gracefully
    assert result in [0, 1]


def test_execute_input_file_split(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test splitting an existing file
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["/etc/passwd"])

    # Should either succeed (splitting /etc/passwd) or fail if file not accessible
    if result == 0:
        # Should have created split files (xaa, xab, etc.)
        pass
    else:
        assert result == 1


def test_execute_nonexistent_input_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test splitting nonexistent file
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/nonexistent/file"])

    # Should fail with file not found error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["No such file", "not found", "error"])


def test_execute_custom_prefix(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test with custom prefix for output files
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["/etc/passwd", "myprefix"])

    # Should either succeed with custom prefix or fail gracefully
    if result == 0:
        # Should have created files with "myprefix" prefix
        pass
    else:
        assert result == 1


def test_execute_lines_option_split_by_lines(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -l option to split by number of lines
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-l", "10", "/etc/passwd"])

    # Should either succeed with 10 lines per file or fail gracefully
    if result == 0:
        # Should have split into files with 10 lines each
        pass
    else:
        assert result == 1


def test_execute_lines_option_invalid_number(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -l option with invalid number
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-l", "invalid", "/etc/passwd"])

    # Should fail with invalid number error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "number"])


def test_execute_bytes_option_split_by_size(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -b option to split by byte size
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-b", "1024", "/etc/passwd"])

    # Should either succeed with 1024 bytes per file or fail gracefully
    if result == 0:
        # Should have split into files with 1024 bytes each
        pass
    else:
        assert result == 1


def test_execute_bytes_option_invalid_size(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -b option with invalid size
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-b", "invalid", "/etc/passwd"])

    # Should fail with invalid size error
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "size"])


def test_execute_suffix_length_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -a option to set suffix length
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-a", "3", "/etc/passwd"])

    # Should either succeed with 3-character suffixes or fail gracefully
    if result == 0:
        # Should have created files with 3-character suffixes (xaaa, xaab, etc.)
        pass
    else:
        assert result == 1


def test_execute_suffix_length_invalid(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -a option with invalid suffix length
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-a", "0", "/etc/passwd"])

    # Should fail with invalid suffix length
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "suffix"])


def test_execute_numeric_suffixes_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -d option for numeric suffixes
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-d", "/etc/passwd"])

    # Should either succeed with numeric suffixes or fail gracefully
    if result == 0:
        # Should have created files with numeric suffixes (x00, x01, etc.)
        pass
    else:
        assert result == 1


def test_execute_verbose_option(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test --verbose option for diagnostic output
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["--verbose", "/etc/passwd"])

    # Should either succeed with verbose output or fail gracefully
    if result == 0:
        _ = capture.get()
        # Should show diagnostic messages about file creation
        pass
    else:
        assert result == 1


def test_execute_line_bytes_option_c(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -C option for maximum bytes per line per file
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-C", "80", "/etc/passwd"])

    # Should either succeed with line-byte limit or fail gracefully
    if result == 0:
        # Should have limited lines to 80 bytes per output file
        pass
    else:
        assert result == 1


def test_execute_multiple_options_combination(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test multiple options together
    with command.shell.console.capture() as _:
        result = command.execute(
            client=client, args=["-l", "5", "-a", "3", "-d", "/etc/passwd", "test"]
        )

    # Should either succeed with all options or fail gracefully
    if result == 0:
        # Should have combined effects: 5 lines, 3-char numeric suffixes, "test" prefix
        pass
    else:
        assert result == 1


def test_execute_size_suffixes_k_m_g(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test size suffixes (k, M, G) if supported
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-b", "1k", "/etc/passwd"])

    # Should either succeed with 1k (1024 bytes) or fail gracefully
    if result == 0:
        # Should have parsed "1k" as 1024 bytes
        pass
    else:
        assert result == 1


def test_execute_zero_lines_split(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -l 0 (should be invalid)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-l", "0", "/etc/passwd"])

    # Should fail with invalid line count
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "zero", "lines"])


def test_execute_zero_bytes_split(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test -b 0 (should be invalid)
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["-b", "0", "/etc/passwd"])

    # Should fail with invalid byte size
    assert result == 1
    output = capture.get()
    assert any(msg in output for msg in ["invalid", "zero", "size"])


def test_execute_output_file_creation(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test that output files are created
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-l", "2", "/etc/passwd"])

    # Should either succeed and create output files or fail gracefully
    if result == 0:
        # Should have created split files in current directory
        pass
    else:
        assert result == 1


def test_execute_empty_input_file(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test splitting empty file (if available)
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["/dev/null"])

    # Should either succeed (creating empty output) or fail gracefully
    if result == 0:
        # Should handle empty input gracefully
        pass
    else:
        assert result == 1


def test_execute_permission_error_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.SplitCommand,
):
    # Test handling of permission errors
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["/etc/shadow"])

    # Should fail gracefully if permission denied
    if result == 1:
        _ = capture.get()
        # Should show appropriate error message
        pass
    else:
        # May succeed if file is readable
        assert result == 0



"""Integration tests for the PstreeCommand."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

import pebble_shell.commands
import pebble_shell.shell

if TYPE_CHECKING:
    import ops


@pytest.fixture
def command(shell: pebble_shell.shell.PebbleShell):
    """Fixture to create a PstreeCommand instance."""
    yield pebble_shell.commands.PstreeCommand(shell=shell)


def test_name(command: pebble_shell.commands.PstreeCommand):
    assert command.name == "pstree"


def test_category(command: pebble_shell.commands.PstreeCommand):
    assert command.category == "System"


def test_help(command: pebble_shell.commands.PstreeCommand):
    with command.shell.console.capture() as capture:
        command.show_help()
    assert "pstree" in capture.get()


@pytest.mark.parametrize("args", [["-h"], ["--help"]])
def test_execute_help(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
    args: list[str],
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=args)
    assert result == 0
    assert "pstree" in capture.get()


def test_execute_no_args_default_tree(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # Should either succeed if /proc is accessible or fail gracefully
    if result == 0:
        # Should contain process tree information
        # At minimum should show some processes
        pass
    else:
        # Should be 1 if no processes found or proc access error
        assert result == 1
        output = capture.get()
        assert any(
            msg in output
            for msg in ["No processes found", "Error building process tree"]
        )


def test_execute_proc_access_error_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # This test verifies that /proc access errors are handled correctly
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # In container environments without full /proc access, should fail gracefully
    if result == 1:
        output = capture.get()
        assert any(
            msg in output
            for msg in ["No processes found", "Error building process tree"]
        )
    else:
        # If it succeeds, should have valid process tree output
        assert result == 0


def test_execute_with_args_ignored(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # pstree command doesn't take arguments besides help, they should be ignored
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=["some", "args"])

    # Should behave same as no arguments
    if result == 0:
        # Should show process tree regardless of extra args
        pass
    else:
        assert result == 1
        output = capture.get()
        assert any(
            msg in output
            for msg in ["No processes found", "Error building process tree"]
        )


def test_execute_tree_structure_validation(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # If successful, should show hierarchical process tree
    if result == 0:
        _ = capture.get()
        # Process tree should contain PIDs and process names
        # Tree structure uses typical tree characters or indentation
        pass
    else:
        assert result == 1


def test_execute_empty_proc_directory(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # Test behavior when /proc has no process directories
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    if result == 1:
        output = capture.get()
        assert "No processes found" in output
    else:
        # If processes are found, should succeed
        assert result == 0


def test_execute_partial_proc_read_errors(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # Test resilience when some process info can't be read
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # Should either succeed (handling partial failures) or fail gracefully
    if result == 0:
        # Command should handle cases where some process info is unreadable
        pass
    else:
        assert result == 1
        output = capture.get()
        assert any(
            msg in output
            for msg in ["No processes found", "Error building process tree"]
        )


def test_execute_process_hierarchy_display(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # If successful, should show parent-child relationships
    if result == 0:
        _ = capture.get()
        # Should contain process hierarchy information
        # May contain process names, PIDs, and tree structure indicators
        pass
    else:
        assert result == 1


def test_execute_root_process_identification(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=[])

    # Should identify and display root processes (PPID 0 or orphaned)
    if result == 0:
        # Tree should start from root processes
        pass
    else:
        assert result == 1


def test_execute_circular_reference_handling(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # Test that circular parent-child references don't cause infinite loops
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=[])

    # Should complete without hanging (visited set prevents infinite recursion)
    if result == 0:
        # Should successfully build tree despite any circular references
        pass
    else:
        assert result == 1


def test_execute_large_process_count(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=[])

    # Should handle systems with many processes efficiently
    if result == 0:
        # Should complete even with large number of processes
        pass
    else:
        assert result == 1


def test_execute_unknown_flags_ignored(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    # Test that unknown flags are ignored (not parsed)
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=["-x", "--unknown"])

    # Should behave same as no arguments (flags are ignored)
    if result == 0:
        # Should show process tree, ignoring the flags
        pass
    else:
        assert result == 1


def test_execute_process_name_display(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as _:
        result = command.execute(client=client, args=[])

    # Should display process names when available
    if result == 0:
        # Output should contain process information including names
        pass
    else:
        assert result == 1


def test_execute_tree_formatting(
    client: ops.pebble.Client,
    command: pebble_shell.commands.PstreeCommand,
):
    with command.shell.console.capture() as capture:
        result = command.execute(client=client, args=[])

    # Should format output as tree structure
    if result == 0:
        _ = capture.get()
        # Tree formatting may use indentation, Unicode tree chars, etc.
        # Should show hierarchical relationships
        pass
    else:
        assert result == 1



"""Implementation of CatvCommand."""

from __future__ import annotations

from typing import TYPE_CHECKING, Union

import ops

from pebble_shell.utils.command_helpers import safe_read_file

from .._base import Command

if TYPE_CHECKING:
    import shimmer

# TODO: Use the prototype from Shimmer.
ClientType = Union[ops.pebble.Client, "shimmer.PebbleCliClient"]


# TODO: Put this in one of the file categories.
class CatvCommand(Command):
    """Display file contents with visible control characters."""

    name = "catv"
    help = "Display file contents with visible control characters"
    category = "File"

    def execute(self, client: ClientType, args: list[str]) -> int:
        """Execute the catv command."""
        return self._execute_catv(client, args)

    def _execute_catv(self, client: ClientType, args: list[str]) -> int:
        if not args:
            self.console.print("Usage: catv [files...]")
            return 1

        for file_path in args:
            try:
                content = safe_read_file(client, file_path, self.shell)
                if content is not None:
                    # Show control characters visibly
                    visible = content.replace("\t", "^I").replace("\n", "$\n")
                    self.console.print(visible, end="")
            except Exception as e:  # noqa: PERF203  # needed for robust file processing
                self.console.print(f"catv: {file_path}: {e}")
                return 1

        return 0



"""Implementation of CpioCommand."""

from __future__ import annotations

import io
import tarfile
from typing import TYPE_CHECKING, Union

import ops

from ...utils.command_helpers import handle_help_flag, parse_flags, safe_read_file
from .._base import Command

if TYPE_CHECKING:
    import shimmer

# TODO: Use the prototype from Shimmer.
ClientType = Union[ops.pebble.Client, "shimmer.PebbleCliClient"]


# TODO: Put this in the compression category.
class CpioCommand(Command):
    """Implementation of cpio command."""

    name = "cpio"
    help = "Copy files to and from archives"
    category = "Archive"

    def execute(self, client: ClientType, args: list[str]) -> int:
        """Execute the cpio command."""
        if handle_help_flag(self, args):
            return 0

        result = parse_flags(
            args,
            {
                "i": bool,  # extract
                "extract": bool,
                "o": bool,  # create
                "create": bool,
                "t": bool,  # list
                "list": bool,
                "v": bool,  # verbose
                "verbose": bool,
                "d": bool,  # make directories
                "make-directories": bool,
                "F": str,  # archive file
                "H": str,  # format
            },
            self.shell,
        )
        if result is None:
            return 1
        flags, positional_args = result

        extract = flags.get("i", False) or flags.get("extract", False)
        create = flags.get("o", False) or flags.get("create", False)
        list_contents = flags.get("t", False) or flags.get("list", False)
        verbose = flags.get("v", False) or flags.get("verbose", False)
        make_dirs = flags.get("d", False) or flags.get("make-directories", False)
        archive_file = flags.get("F")
        format_type = flags.get("H", "newc")

        # Exactly one mode must be specified
        mode_count = sum([extract, create, list_contents])
        if mode_count != 1:
            self.console.print(
                "[red]cpio: must specify exactly one of -i, -o, or -t[/red]"
            )
            return 1

        try:
            if list_contents:
                return self._list_archive(client, archive_file, verbose)
            elif extract:
                return self._extract_archive(client, archive_file, verbose, make_dirs)
            elif create:
                return self._create_archive(client, archive_file, verbose, format_type)
            else:
                return 1

        except Exception as e:
            self.console.print(f"[red]cpio: {e}[/red]")
            return 1

    def _list_archive(
        self, client: ClientType, archive_file: str, verbose: bool
    ) -> int:
        """List contents of an archive."""
        if not archive_file:
            self.console.print("[red]cpio: archive file required for listing[/red]")
            return 1

        try:
            # Read archive file
            archive_content = safe_read_file(client, archive_file, self.shell)
            if archive_content is None:
                self.console.print(
                    f"[red]cpio: cannot read archive '{archive_file}'[/red]"
                )
                return 1

            # Try to read as tar format (most common)
            try:
                with tarfile.open(
                    fileobj=io.BytesIO(archive_content.encode()), mode="r"
                ) as tar:
                    for member in tar.getmembers():
                        if verbose:
                            self.console.print(
                                f"{member.mode:o} {member.uid}/{member.gid} {member.size:8d} {member.name}"
                            )
                        else:
                            self.console.print(member.name)
                return 0
            except tarfile.TarError:
                self.console.print(
                    "[yellow]cpio: archive format not supported for listing[/yellow]"
                )
                return 1

        except Exception as e:
            self.console.print(f"[red]cpio: error listing archive: {e}[/red]")
            return 1

    def _extract_archive(
        self, client: ClientType, archive_file: str, verbose: bool, make_dirs: bool
    ) -> int:
        """Extract files from an archive."""
        if not archive_file:
            self.console.print("[red]cpio: archive file required for extraction[/red]")
            return 1

        try:
            # Read archive file
            archive_content = safe_read_file(client, archive_file, self.shell)
            if archive_content is None:
                self.console.print(
                    f"[red]cpio: cannot read archive '{archive_file}'[/red]"
                )
                return 1

            # Try to extract as tar format
            try:
                with tarfile.open(
                    fileobj=io.BytesIO(archive_content.encode()), mode="r"
                ) as tar:
                    extracted_count = 0
                    for member in tar.getmembers():
                        if member.isfile():
                            file_content = tar.extractfile(member)
                            if file_content:
                                # Create directories if needed
                                if make_dirs and "/" in member.name:
                                    dir_path = "/".join(member.name.split("/")[:-1])
                                    # Note: In a real implementation, we'd need to create directories
                                    # For this demo, we'll just note it
                                    if verbose:
                                        self.console.print(
                                            f"[dim]Would create directory: {dir_path}[/dim]"
                                        )

                                # Write file (simulate)
                                if verbose:
                                    self.console.print(f"Extracting: {member.name}")

                                # In a real implementation, we'd write to the remote filesystem
                                # For this demo, we'll just count the files
                                extracted_count += 1

                    self.console.print(
                        f"[green]Extracted {extracted_count} files[/green]"
                    )
                    return 0
            except tarfile.TarError:
                self.console.print(
                    "[yellow]cpio: archive format not supported for extraction[/yellow]"
                )
                return 1

        except Exception as e:
            self.console.print(f"[red]cpio: error extracting archive: {e}[/red]")
            return 1

    def _create_archive(
        self, client: ClientType, archive_file: str, verbose: bool, format_type: str
    ) -> int:
        """Create an archive from file list."""
        self.console.print(
            "[yellow]cpio: creating archives not implemented in read-only mode[/yellow]"
        )
        self.console.print(
            "[dim]Would read file list from stdin and create archive[/dim]"
        )
        return 0
